import { generateSchemaExample } from './generateSchemaExample';
import { json2xml } from './json2xml';
import { stringifyOpenAPI } from './stringifyOpenAPI';
import { checkIsReference } from './utils';
/**
 * Display an example.
 */
export function OpenAPIExample(props) {
    var example = props.example, context = props.context, syntax = props.syntax;
    var code = stringifyExample({ example: example, xml: syntax === 'xml' });
    if (code === null) {
        return <OpenAPIEmptyExample />;
    }
    return context.renderCodeBlock({ code: code, syntax: syntax });
}
function stringifyExample(args) {
    var example = args.example, xml = args.xml;
    if (!example.value) {
        return null;
    }
    if (typeof example.value === 'string') {
        return example.value;
    }
    if (xml) {
        return json2xml(example.value);
    }
    return stringifyOpenAPI(example.value, null, 2);
}
/**
 * Empty response example.
 */
export function OpenAPIEmptyExample() {
    return (<pre className="openapi-example-empty">
            <p>No Content</p>
        </pre>);
}
/**
 * Generate an example from a reference object.
 */
export function getExampleFromReference(ref) {
    return { summary: 'Unresolved reference', value: { $ref: ref.$ref } };
}
/**
 * Get examples from a media type object.
 */
export function getExamplesFromMediaTypeObject(args) {
    var _a;
    var _b, _c;
    var mediaTypeObject = args.mediaTypeObject, mediaType = args.mediaType;
    if (mediaTypeObject.examples) {
        return Object.entries(mediaTypeObject.examples).map(function (_a) {
            var key = _a[0], example = _a[1];
            return {
                key: key,
                example: checkIsReference(example) ? getExampleFromReference(example) : example,
            };
        });
    }
    if (mediaTypeObject.example) {
        return [{ key: 'default', example: { value: mediaTypeObject.example } }];
    }
    if (mediaTypeObject.schema) {
        if (mediaType === 'application/xml') {
            // @TODO normally we should use the name of the schema but we don't have it
            // fix it when we got the reference name
            var root = (_c = (_b = mediaTypeObject.schema.xml) === null || _b === void 0 ? void 0 : _b.name) !== null && _c !== void 0 ? _c : 'object';
            return [
                {
                    key: 'default',
                    example: {
                        value: (_a = {},
                            _a[root] = generateSchemaExample(mediaTypeObject.schema, {
                                xml: mediaType === 'application/xml',
                                mode: 'read',
                            }),
                            _a),
                    },
                },
            ];
        }
        return [
            {
                key: 'default',
                example: {
                    value: generateSchemaExample(mediaTypeObject.schema, {
                        mode: 'read',
                    }),
                },
            },
        ];
    }
    return [];
}
/**
 * Get example from a schema object.
 */
export function getExampleFromSchema(args) {
    var schema = args.schema;
    if (schema.example) {
        return { value: schema.example };
    }
    return { value: generateSchemaExample(schema, { mode: 'read' }) };
}
