import { Markdown } from './Markdown';
import { OpenAPIEmptyExample, OpenAPIExample, getExampleFromReference, getExamplesFromMediaTypeObject, } from './OpenAPIExample';
import { OpenAPITabs, OpenAPITabsList, OpenAPITabsPanels } from './OpenAPITabs';
import { StaticSection } from './StaticSection';
import { checkIsReference, createStateKey, resolveDescription } from './utils';
/**
 * Display an example of the response content.
 */
export function OpenAPIResponseExample(props) {
    var data = props.data, context = props.context;
    // if there are no responses defined for the operation
    if (!data.operation.responses) {
        return null;
    }
    var responses = Object.entries(data.operation.responses);
    // Sort response to get 200, and 2xx first
    responses.sort(function (_a, _b) {
        var a = _a[0];
        var b = _b[0];
        if (a === 'default') {
            return 1;
        }
        if (b === 'default') {
            return -1;
        }
        if (a === '200') {
            return -1;
        }
        if (b === '200') {
            return 1;
        }
        return Number(a) - Number(b);
    });
    var tabs = responses.map(function (_a) {
        var key = _a[0], responseObject = _a[1];
        var description = resolveDescription(responseObject);
        if (checkIsReference(responseObject)) {
            return {
                key: key,
                label: key,
                body: (<OpenAPIExample example={getExampleFromReference(responseObject)} context={context} syntax="json"/>),
                footer: description ? <Markdown source={description}/> : undefined,
            };
        }
        if (!responseObject.content || Object.keys(responseObject.content).length === 0) {
            return {
                key: key,
                label: key,
                body: <OpenAPIEmptyExample />,
                footer: description ? <Markdown source={description}/> : undefined,
            };
        }
        return {
            key: key,
            label: key,
            body: <OpenAPIResponse context={context} content={responseObject.content}/>,
            footer: description ? <Markdown source={description}/> : undefined,
        };
    });
    if (tabs.length === 0) {
        return null;
    }
    return (<OpenAPITabs stateKey={createStateKey('response-example')} items={tabs}>
            <StaticSection header={<OpenAPITabsList />} className="openapi-panel">
                <OpenAPITabsPanels />
            </StaticSection>
        </OpenAPITabs>);
}
function OpenAPIResponse(props) {
    var context = props.context, content = props.content;
    var entries = Object.entries(content);
    var firstEntry = entries[0];
    if (!firstEntry) {
        throw new Error('One media type is required');
    }
    if (entries.length === 1) {
        var mediaType = firstEntry[0], mediaTypeObject = firstEntry[1];
        return (<OpenAPIResponseMediaType context={context} mediaType={mediaType} mediaTypeObject={mediaTypeObject}/>);
    }
    var tabs = entries.map(function (entry) {
        var mediaType = entry[0], mediaTypeObject = entry[1];
        return {
            key: mediaType,
            label: mediaType,
            body: (<OpenAPIResponseMediaType context={context} mediaType={mediaType} mediaTypeObject={mediaTypeObject}/>),
        };
    });
    return (<OpenAPITabs stateKey={createStateKey('response-media-types')} items={tabs}>
            <StaticSection header={<OpenAPITabsList />} className="openapi-response-media-types">
                <OpenAPITabsPanels />
            </StaticSection>
        </OpenAPITabs>);
}
function OpenAPIResponseMediaType(props) {
    var mediaTypeObject = props.mediaTypeObject, mediaType = props.mediaType;
    var examples = getExamplesFromMediaTypeObject({ mediaTypeObject: mediaTypeObject, mediaType: mediaType });
    var syntax = getSyntaxFromMediaType(mediaType);
    var firstExample = examples[0];
    if (!firstExample) {
        return <OpenAPIEmptyExample />;
    }
    if (examples.length === 1) {
        return (<OpenAPIExample example={firstExample.example} context={props.context} syntax={syntax}/>);
    }
    var tabs = examples.map(function (example) {
        return {
            key: example.key,
            label: example.example.summary || example.key,
            body: (<OpenAPIExample example={example.example} context={props.context} syntax={syntax}/>),
        };
    });
    return (<OpenAPITabs stateKey={createStateKey('response-media-type-examples')} items={tabs}>
            <StaticSection header={<OpenAPITabsList />} className="openapi-response-media-type-examples">
                <OpenAPITabsPanels />
            </StaticSection>
        </OpenAPITabs>);
}
/**
 * Get the syntax from a media type.
 */
function getSyntaxFromMediaType(mediaType) {
    if (mediaType.includes('json')) {
        return 'json';
    }
    if (mediaType === 'application/xml') {
        return 'xml';
    }
    return 'text';
}
